
clear
clc

%% Raw data path
TrainImagePath = 'E:\DATABASE\EmotiW2013\AFEW\Train\';
TrainAudioPath = 'E:\DATABASE\EmotiW2013\AudioFeatures\Train\';
ValImagePath = 'E:\DATABASE\EmotiW2013\AFEW\Val\';
ValAudioPath = 'E:\DATABASE\EmotiW2013\AudioFeatures\Val\';
SubDir = {'Angry\','Disgust\','Fear\','Happy\','Neutral\', 'Sad\','Surprise\'};

%% Experimental paremeters
fprintf('Experimental paremeters:\n');
alpha = 2^6;
ncomp_V = 10;
ncomp_A = 5;
lambda = 0.35;
fprintf('--The fusion weight of Grassmann kernel: alpha = %d\n',alpha);
fprintf('--The dimensions of One-to-rest PLS for video: ncomp_V = %d\n',ncomp_V);
fprintf('--The dimensions of One-to-rest PLS for audio: ncomp_A = %d\n',ncomp_A);
fprintf('--The video-audio fusion weight: lambda = %.2f\n',lambda);

%% Parsing raw data and label
fprintf('Parsing raw data and label.\n');

DataGrayPath = '.\data\Data_Gray_32x32_Train_Val.mat';
LabelPath = '.\data\Label_Train_Val.mat';
if exist(DataGrayPath,'file')
    load(DataGrayPath);
    load(LabelPath);
else
    [TrainData,TrainLabel] = ParseImageData(TrainImagePath,SubDir);
    [ValData,ValLabel] = ParseImageData(ValImagePath,SubDir);
    save(DataGrayPath,'TrainData','ValData');
    save(LabelPath,'TrainLabel','ValLabel');
end
fprintf('--The number of train clips: %d\n',length(TrainLabel));
fprintf('--The number of validation clips: %d\n',length(ValLabel));

%% Training image feature template
fprintf('Training image feature template.\n');

[Paras,item] = Configure;
FeatureTemplatePath = sprintf('.\\data\\FeatureTemplate_Filters_%dx%d_%d_W_%d.mat',...
    Paras.kernelH,Paras.kernelW,Paras.numMaps,item.PCADim);
if exist(FeatureTemplatePath,'file')
    load(FeatureTemplatePath);
else  
    [Filters,W] = FeatureTemplate(TrainData,Paras,item);
    save(FeatureTemplatePath,'Filters','W');
end
fprintf('--The size of filters: %dx%d\n',Paras.kernelH,Paras.kernelW);
fprintf('--The number of filters: %d\n',Paras.numMaps);


%% Extracting image feature
fprintf('Extracting image feature.\n');

DataFGPCAPath = sprintf('.\\data\\Data_FG_%d_PCA_%d_Train_Val.mat',Paras.ConvFeaDim,item.PCADim);
if exist(DataFGPCAPath,'file')
    load(DataFGPCAPath);
else
    TrainImageFeature = ImageFeature(TrainData,Filters,Paras,W);
    ValImageFeature = ImageFeature(ValData,Filters,Paras,W);
    save(DataFGPCAPath,'TrainImageFeature','ValImageFeature');
end
fprintf('--The convolution feature dim: %d\n',Paras.ConvFeaDim);
fprintf('--The PCA reduction dim: %d\n',item.PCADim);

%% Extracting Video feature
fprintf('Extracting Video feature.\n');

fprintf('--Computing Grassmann kernel.\n');
GGDAKernelPath = '.\data\Data_Kernel_GGDA_Train_Val.mat';
SubspacePath = '.\data\Data_Subspace_Train_Val.mat';
if exist(GGDAKernelPath,'file')
    load(GGDAKernelPath);
else
    TrainSubspace = Compute_Subspace(TrainImageFeature);
    ValSubspace = Compute_Subspace(ValImageFeature);
    save(SubspacePath,'TrainSubspace','ValSubspace');
    TrainKernel = Compute_Grassmann_Kernel(TrainSubspace,[]);
    ValKernel = Compute_Grassmann_Kernel(TrainSubspace,ValSubspace); 
    save(GGDAKernelPath,'TrainKernel','ValKernel');
end

fprintf('--Fusion kernels as video feature.\n');
DataVideoPath = '.\data\Data_Video_Train_Val.mat';
if exist(DataVideoPath,'file')
    load(DataVideoPath);
else    
    TrainVideoFeature = VideoFeature(TrainKernel,alpha);
    ValVideoFeature = VideoFeature(ValKernel,alpha);
    save(DataVideoPath,'TrainVideoFeature','ValVideoFeature');
end
fprintf('--The dimensions of video feature: %d\n',size(TrainVideoFeature,2));

%% Parsing audio feature
fprintf('Parsing audio feature.\n');

DataAudioPath = '.\data\Data_Audio_Train_Val.mat';
if exist(DataAudioPath,'file')
    load(DataAudioPath);
else
    TrainAudioFeature = AudioFeature(TrainAudioPath,SubDir);
    ValAudioFeature = AudioFeature(ValAudioPath,SubDir);
    save(DataAudioPath,'TrainAudioFeature','ValAudioFeature');
end
fprintf('--The dimensions of audio feature: %d\n',size(TrainAudioFeature,2));

%% One-to-rest PLS on audio and video data
fprintf('One-to-rest PLS on audio and video data.\n');

ValidIdx = logical(ValidSet(TrainImageFeature));
[~,Acc_V,~,Fit_V] = OneToRestPLS(TrainVideoFeature(ValidIdx,ValidIdx),TrainLabel(ValidIdx),ValVideoFeature(:,ValidIdx),ValLabel,ncomp_V);
[~,Acc_A,~,Fit_A] = OneToRestPLS(TrainAudioFeature(ValidIdx,:),TrainLabel(ValidIdx),ValAudioFeature,ValLabel,ncomp_A);
fprintf('--The performance of video feature: %.4f\n',Acc_V);
fprintf('--The performance of audio feature: %.4f\n',Acc_A);

%% Decision-level fusion
fprintf('Decision-level fusion.\n');

Fit = lambda*Fit_A+(1-lambda)*Fit_V;
[~,Predict] = max(Fit,[],2);
Acc = length(find(Predict==ValLabel))/length(ValLabel);
fprintf('--The fusion performance on validation set: %.4f\n', Acc);
