clear
clc

[Paras,item] = Configure;
DataFGPCAPath = sprintf('.\\data\\Data_FG_%d_PCA_%d_Train_Val.mat',Paras.ConvFeaDim,item.PCADim);
load(DataFGPCAPath);
SubspacePath = '.\data\Data_Subspace_Train_Val.mat';
load(SubspacePath);

DataVideoPath = '.\data\Data_Video_Train_Val.mat';
DataAudioPath = '.\data\Data_Audio_Train_Val.mat';
LabelPath = '.\data\Label_Train_Val.mat';
load(DataVideoPath);
load(DataAudioPath);
load(LabelPath);

%% Raw data path
TestImagePath = '.\test\Video\';
TestAudioPath = '.\test\Audio\';

for TestIdx = 1:312
% TestIdx = input('Input the index of test sample (range from 1 to 312): \n');
fprintf('The TestIdx: %d\n',TestIdx);
%% Experimental paremeters
fprintf('Experimental paremeters:\n');
alpha = 2^6;
ncomp_V = 10;
ncomp_A = 5;
lambda = 0.35;
fprintf('--The fusion weight of Grassmann kernel: alpha = %d\n',alpha);
fprintf('--The dimensions of One-to-rest PLS for video: ncomp_V = %d\n',ncomp_V);
fprintf('--The dimensions of One-to-rest PLS for audio: ncomp_A = %d\n',ncomp_A);
fprintf('--The video-audio fusion weight: lambda = %.2f\n',lambda);

%% Parsing and displaying test images
fprintf('Parsing and displaying test images.\n');

dirVideo = dir([TestImagePath '0*']);
dirImage = dir([TestImagePath dirVideo(TestIdx).name '\*.png']);
if length(dirImage)>=1
    TMP = [];
%     figure,
    for k = 1:length(dirImage)
        Img = imread([TestImagePath dirVideo(TestIdx).name '\' dirImage(k).name]);
%         imshow(Img);
%         pause(0.1);
        Img = rgb2gray(Img);
        Img = imresize(Img,[32 32]);
        TMP(k,:) = Img(:);
    end
    TestData{1,1} = TMP'; 
else
    error('No image in the test video clip!')
end

%% loading feature template and extract image feature
fprintf('loading feature template and extract image feature.\n');

[Paras,item] = Configure;
FeatureTemplatePath = sprintf('.\\data\\FeatureTemplate_Filters_%dx%d_%d_W_%d.mat',...
    Paras.kernelH,Paras.kernelW,Paras.numMaps,item.PCADim);
load(FeatureTemplatePath);
TestImageFeature = ImageFeature(TestData,Filters,Paras,W);

%% Extracting video feature
fprintf('Extracting video feature.\n');

% DataFGPCAPath = sprintf('.\\data\\Data_FG_%d_PCA_%d_Train_Val.mat',Paras.ConvFeaDim,item.PCADim);
% load(DataFGPCAPath);
% SubspacePath = '.\data\Data_Subspace_Train_Val.mat';
% load(SubspacePath);
TestSubspace = Compute_Subspace(TestImageFeature);
TestKernel = Compute_Grassmann_Kernel(TrainSubspace,TestSubspace);
TestVideoFeature = VideoFeature(TestKernel,alpha);

%% Parsing audio feature
fprintf('Parsing audio feature.\n');
dirAudio = dir([TestAudioPath '0*']);
filename = [TestAudioPath dirAudio(TestIdx).name];
TestAudioFeature = ParseARFF(filename);

%% One-to-rest PLS on audio and video data
fprintf('One-to-rest PLS on audio and video data.\n');

ValidIdx = logical(ValidSet(TrainImageFeature));
% DataVideoPath = '.\data\Data_Video_Train_Val.mat';
% DataAudioPath = '.\data\Data_Audio_Train_Val.mat';
% LabelPath = '.\data\Label_Train_Val.mat';
% load(DataVideoPath);
% load(DataAudioPath);
% load(LabelPath);
[~,~,~,Fit_V] = OneToRestPLS(TrainVideoFeature(ValidIdx,ValidIdx),TrainLabel(ValidIdx),TestVideoFeature(:,ValidIdx),0,ncomp_V);
[~,~,~,Fit_A] = OneToRestPLS(TrainAudioFeature(ValidIdx,:),TrainLabel(ValidIdx),TestAudioFeature,0,ncomp_A);

%% Decision-level fusion
fprintf('Decision-level fusion.\n');

Fit = lambda*Fit_A+(1-lambda)*Fit_V;
[~,Predict] = max(Fit,[],2);
enum = {'Angry','Disgust','Fear','Happy','Neutral', 'Sad','Surprise'};
fprintf('--The fusion result is: %s\n', enum{Predict});

Results{TestIdx} = enum{Predict};

end
