function ConvData = ConvPooling(Data,Filters,Paras)

[numMaps,~] = size(Filters.centroids);
Paras.pImgH = floor((Paras.imgH - Paras.kernelH) / Paras.stride) + 1;
Paras.pImgW = floor((Paras.imgW - Paras.kernelW) / Paras.stride) + 1;
pFeaDim = floor(Paras.pImgH / Paras.poolH) * floor(Paras.pImgW / Paras.poolW);
ConvData = zeros(size(Data,1),numMaps * pFeaDim);

for i = 1:size(Data,1)
    Img = reshape(Data(i,:), Paras.imgH, Paras.imgW);
    patches = im2col(Img,[Paras.kernelH, Paras.kernelW])';
    patches = bsxfun(@rdivide, bsxfun(@minus, patches, mean(patches,2)), sqrt(var(patches,[],2)+10));
    patches = bsxfun(@minus, patches, Filters.mean) * Filters.proj;
    convImg = Filters.centroids * patches';
    pConvImg = MaxPooling(convImg,Paras);
    ConvData(i,:) = pConvImg;
end

function PoolImgSet = MaxPooling(ImgSet,Paras)

numMaps = size(ImgSet,1);
poolImg = zeros(floor(Paras.pImgH / Paras.poolH), floor(Paras.pImgW / Paras.poolW));
pFeaDim = numel(poolImg);
PoolImgSet = zeros(pFeaDim * numMaps,1);

for m = 1:numMaps
    Img = reshape(ImgSet(m,:)',[Paras.pImgH Paras.pImgW]);
    for locateH = 1 : Paras.poolH : Paras.pImgH - mod(Paras.pImgH, Paras.poolH)
        for locateW = 1 : Paras.poolW : Paras.pImgW - mod(Paras.pImgW, Paras.poolW)
            pool = Img(locateH : locateH + Paras.poolH - 1, locateW : locateW + Paras.poolW - 1);
            poolImg(ceil(locateH / Paras.poolH),ceil(locateW / Paras.poolW)) = max(max(pool));
        end
    end
    PoolImgSet(pFeaDim*(m-1)+1:pFeaDim*m) = poolImg(:);
end




